import { Never } from 'javascriptutilities';
import { MappableObserver } from 'rx-utilities-js';
import { Observable, Scheduler } from 'rxjs';
import { Type as StoreType } from './types';
export declare namespace Action {
    /**
     * Represents an action to be dispatched to the global state.
     * @template T Generics parameter.
     */
    interface Type<T> {
        readonly id: string;
        readonly fullValuePath: string;
        readonly payload: T;
    }
}
export declare type Reducer<State, T> = (state: State, action: Action.Type<T>) => State;
/**
 * Represents a dispatch store type.
 * @extends {StoreType} Store type extension.
 * @template State Generics parameter.
 */
export interface Type<State> extends StoreType<State> {
    readonly actionTrigger: MappableObserver.Type<Never<Action.Type<any>>>;
    readonly actionStream: Observable<Action.Type<any>>;
    readonly lastState: State;
    dispatch(action: Action.Type<any>): void;
}
/**
 * Act as a centralized storage for state. This store can dispatch actions that
 * will be reduced onto the existing state.
 * @implements {Type} Type implementation.
 * @template State Generics parameter.
 */
export declare class Impl<State> implements Type<State> {
    private readonly action;
    private readonly state;
    private readonly subscription;
    constructor(initialState: State);
    readonly actionTrigger: MappableObserver.Type<Never<Action.Type<any>>>;
    readonly actionStream: Observable<Action.Type<any>>;
    readonly stateStream: Observable<State>;
    readonly lastState: State;
    initialize(reducer: Reducer<State, any>, scheduler: Scheduler): void;
    deinitialize(): void;
    dispatch(action: Action.Type<any>): void;
}
/**
 * Create and initialize a dispatch store.
 * @template State Generics parameter.
 * @param {State} initialState Initial state.
 * @param {Reducer<State, any>} reducer A Reducer instance.
 * @param {Scheduler} scheduler A Scheduler instance.
 * @returns {Impl} A dispatch store instance.
 */
export declare function createDefault<State>(initialState: State, reducer: Reducer<State, any>, scheduler: Scheduler): Impl<State>;
